"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isEqualCheck = exports.memoize = exports.Memoize = void 0;
/**
 * Memoize decorator
 * @constructor
 */
function Memoize() {
    return function (target, propertyKey, descriptor) {
        const originalMethod = descriptor.value;
        //wrapping the original method
        descriptor.value = function (...args) {
            const memoizer = memoize(originalMethod);
            return memoizer.memoized(...args);
        };
    };
}
exports.Memoize = Memoize;
/**
 * Memoize function
 * @param projectionFn
 * @param comparatorFn
 */
function memoize(projectionFn, comparatorFn) {
    return resultMemoize(projectionFn, comparatorFn !== null && comparatorFn !== void 0 ? comparatorFn : defaultComparatorFn);
}
exports.memoize = memoize;
function defaultComparatorFn(a, b) {
    if (a instanceof Array) {
        return a.length === b.length && a.every((fromA) => b.includes(fromA));
    }
    // Default comparison
    return a === b;
}
function isEqualCheck(a, b) {
    return a === b;
}
exports.isEqualCheck = isEqualCheck;
function isArgumentsChanged(args, lastArguments, comparator) {
    for (let i = 0; i < args.length; i++) {
        if (!comparator(args[i], lastArguments[i])) {
            return true;
        }
    }
    return false;
}
function resultMemoize(projectionFn, isResultEqual) {
    return defaultMemoize(projectionFn, isEqualCheck, isResultEqual);
}
function defaultMemoize(projectionFn, isArgumentsEqual = isEqualCheck, isResultEqual = isEqualCheck) {
    let lastArguments = null;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any, , , , ,
    let lastResult = null;
    let overrideResult;
    function reset() {
        lastArguments = null;
        lastResult = null;
        overrideResult = undefined;
    }
    function setResult(result = undefined) {
        overrideResult = { result };
    }
    /* eslint-disable prefer-rest-params, prefer-spread */
    // disabled because of the use of `arguments`
    function memoized() {
        if (overrideResult !== undefined) {
            return overrideResult.result;
        }
        if (!lastArguments) {
            lastResult = projectionFn.apply(null, arguments);
            lastArguments = arguments;
            return lastResult;
        }
        if (!isArgumentsChanged(arguments, lastArguments, isArgumentsEqual)) {
            return lastResult;
        }
        const newResult = projectionFn.apply(null, arguments);
        lastArguments = arguments;
        if (isResultEqual(lastResult, newResult)) {
            return lastResult;
        }
        lastResult = newResult;
        return newResult;
    }
    return { memoized, reset, setResult };
}
//# sourceMappingURL=ts-memoize.js.map